# Differential Expression and Pathway Analysis for Seq2Fun
# June 9, 2020
# Jessica Ewald

setwd("/home/peng/test/test/Case_study_2021")

# HTSanalyzeR can no longer be installed for current versions of R
# Here, we provide R source function from HTSanalyzeR package that are needed to run the code
HTS.files <- list.files("HTSanalyzeR", full.names = TRUE)
for(i in c(1:length(HTS.files))){
  source(HTS.files[i])
}

library(dplyr)
library(reshape)
library(ggplot2)
library(ggridges)

# get differential expression analysis results
degs <- readRDS("dea_res.rds")

degs$maxFC <- apply(degs[,c(1,2)], 1, function(x){max(abs(x))})

sig.genes <- degs[degs$adj.P.Val < 0.05 & degs$maxFC > 1, ] %>% rownames()

# get pathways
kegg <- readRDS("kegg.rds")
kegg <- kegg$sets
gs.list <- list(kegg = kegg)
pw.names <- read.table("kegg_pw_names.txt",
                       sep = "\t")

# do gene set analysis
### NOTE: GSEA relies on random permutations. Since we do a relatively low number (n = 1000)
### to improve the speed for demonstration purposes, you can expect to get slightly different 
### results than the case study figure on seq2fun.ca.
degs.maxFC <- degs$maxFC
names(degs.maxFC) <- rownames(degs)

results <- analyzeGeneSetCollections(gs.list, degs.maxFC, sig.genes, 
                                     pAdjustMethod="BH", pValueCutoff=0.05, nPermutations=1000, 
                                     minGeneSetSize=5, exponent=1)

# gsoa results
res.gsoa <- results$HyperGeo.results$kegg
res.gsoa <- merge(res.gsoa, pw.names, by.x = "row.names", by.y = "V1", all = FALSE)
colnames(res.gsoa) <- c("map", "universe.size", "set.size", "total.hits", "exp.hits", 
                        "obs.hits", "pval", "adj.pval", "name")

# gsea results
res.gsea <- results[["GSEA.results"]][["kegg"]]
res.gsea <- merge(res.gsea, pw.names, by.x = "row.names", by.y = "V1", all = FALSE)
colnames(res.gsea) <- c("map", "obs.score", "pval", "adj.pval", "fdr", "name")

# get sig gsea results
gsea.sig <- res.gsea[res.gsea$adj.pval < 0.05, c(1,4,6)]

# prepare data for plotting
degs$KO <- rownames(degs)
degs.plot <- degs[,c(8,1,2)]
degs.plot <- melt(degs.plot)

kegg.plot <- melt(kegg)
colnames(kegg.plot) <- c("KO", "map")

df <- merge(gsea.sig, kegg.plot, by = "map", all.x = TRUE, all.y = FALSE)
df <- merge(df, degs.plot, by = "KO", all.x = TRUE, all.y = FALSE)
df <- na.omit(df)

# calculate the mean fold change to order the pathways in the plot
means <- aggregate(df$value, by = list(df$name), mean)
means <- means[order(means$x, decreasing = FALSE), ]
df$name <- factor(df$name, levels = means$Group.1)

# make the plot
ggplot(df, aes(x = value, y = name, fill = adj.pval)) +
  geom_density_ridges(
    jittered_points = TRUE, point_shape = "|", point_size = 2, point_color = "#898A89",
    color = "white",
    scale = 2, rel_min_height = .01, size = 0.25,
    position = position_points_jitter(height = 0)) +
  scale_y_discrete(expand = c(0, 0), name = "KEGG pathway") +
  scale_x_continuous(expand = c(0, 0), name = "KO log2FC") +
  scale_fill_gradient("Adj. p-value",
                      low = '#ffa34d',
                      high = "#994a00")+
  coord_cartesian(clip = "off") +
  theme_ridges(center = TRUE) +
  theme(legend.position = "none",
        text = element_text(size=12, color = "black"),
        axis.title = element_text(size=12, face = "bold"),
        axis.text.x = element_text(color = "black"),
        axis.text.y = element_text(color = "black"))