# Differential Expression and Pathway Analysis for Seq2Fun using fgsea for pathway analysis
# April 7, 2021
# Jessica Ewald

setwd("/home/peng/test/test/Case_study_2021")
library(dplyr)
library(fgsea)
library(reshape)
library(ggplot2)
library(ggridges)

# get differential expression analysis results
degs <- readRDS("dea_res.rds")

degs$maxFC <- apply(degs[,c(1,2)], 1, function(x){max(abs(x))})

sig.genes <- degs[degs$adj.P.Val < 0.05 & degs$maxFC > 1, ] %>% rownames()

# get pathways
kegg <- readRDS("kegg.rds")
kegg <- kegg$sets
gs.list <- list(kegg = kegg)
pw.names <- read.table("kegg_pw_names.txt",
                       sep = "\t")

# do gene set analysis
degs.maxFC <- degs$maxFC
names(degs.maxFC) <- rownames(degs)
degs.maxFC <- degs.maxFC[order(degs.maxFC)]

# calcualte gsea results
res.gsea <- fgsea(pathways = kegg, 
                  stats    = degs.maxFC,
                  minSize  = 5,
                  scoreType = "pos")
res.gsea <- merge(pw.names, res.gsea, by.x = "V1", by.y = "pathway", all = FALSE)
res.gsea <- res.gsea[order(res.gsea$pval), ]

# calculate ora results
res.gsoa <- fora(pathways = kegg,
                genes = sig.genes,
                universe = names(degs.maxFC),
                minSize = 10,
                maxSize = 500)
res.gsoa <- merge(pw.names, res.gsoa, by.y = "pathway", by.x = "V1", all = FALSE)
res.gsoa <- res.gsoa[order(res.gsoa$pval), ]

# gsoa results
res.gsoa <- res.gsoa[,c(1:4)]
colnames(res.gsoa) <- c("map", "name", "pval", "adj.pval")


# gsea results
res.gsea <- res.gsea[,c(1:4)]
colnames(res.gsea) <- c("map", "name", "pval", "adj.pval")

# get sig gsea results
gsea.sig <- res.gsea[res.gsea$adj.pval < 0.05, ]

# prepare data for plotting
degs$KO <- rownames(degs)
degs.plot <- degs[,c(8,1,2)]
degs.plot <- melt(degs.plot)

kegg.plot <- melt(kegg)
colnames(kegg.plot) <- c("KO", "map")

df <- merge(gsea.sig, kegg.plot, by = "map", all.x = TRUE, all.y = FALSE)
df <- merge(df, degs.plot, by = "KO", all.x = TRUE, all.y = FALSE)
df <- na.omit(df)

# calculate the mean fold change to order the pathways in the plot
means <- aggregate(df$value, by = list(df$name), mean)
means <- means[order(means$x, decreasing = FALSE), ]
df$name <- factor(df$name, levels = means$Group.1)

# make the plot
ggplot(df, aes(x = value, y = name, fill = adj.pval)) +
  geom_density_ridges(
    jittered_points = TRUE, point_shape = "|", point_size = 2, point_color = "#898A89",
    color = "white",
    scale = 2, rel_min_height = .01, size = 0.25,
    position = position_points_jitter(height = 0)) +
  scale_y_discrete(expand = c(0, 0), name = "KEGG pathway") +
  scale_x_continuous(expand = c(0, 0), name = "KO log2FC") +
  scale_fill_gradient("Adj. p-value",
                      low = '#ffa34d',
                      high = "#994a00")+
  coord_cartesian(clip = "off") +
  theme_ridges(center = TRUE) +
  theme(legend.position = "none",
        text = element_text(size=12, color = "black"),
        axis.title = element_text(size=12, face = "bold"),
        axis.text.x = element_text(color = "black"),
        axis.text.y = element_text(color = "black"))
